% std_backproj - backprojects cluster IC ERPs to each subject's channel(s), or backprojects
%                the ICs of the selected clusters to each setfiles and seve it as a new setfile
%                when checkbox is on. Backprojected data are located at STUDY.backProj.backProjectedData
%                where {groups} and [channel, data, subjects, conditions] that corresponds to 
%                STUDY.backProj.backProjSubjList. LIMITATION: separate setfiles for 
%                within-subject conditions not supported. Session not supported.
%
% Usage:
% [STUDY backProjectedData time channel cluster uniqSubjGroupList completeList] = ...
%                                           std_backproj(STUDY, ALLEEG, EEG)
% Inputs:
% STUDY, ALLEEG, EEG: EEGLAB variables.
% GUI               : In the GUI, input channel labels as Fz Cz Pz using
%                     space as a separator. Allows to enter multiple
%                     labels. Must be EXACTLY the same letter strings (case
%                     sensitive).
% Outputs:
%             STUDY: EEGLAB STUDY structure.
% backProjectedData: size [condition group]. If only one channel is
%                    selected, then each cell sizes [timepoints subjects].
%                    Otherwise it is [channel timepoints subjects].
%             times: ERP latencies [ms].
%           channel: input channels.
%           cluster: input clusters.
% uniqSubjGroupList: column1 == included subjects, column2 == group index
%      completeList: column1 == subjects, column2 == IC index, column3 == group index.
% 
% Note: This function internally calls strsplit.m written by Levente Hunyadi,
%       Copyright 2008-2009.

% Author: Makoto Miyakoshi JSPS/SCCN,INC,UCSD
% History:
% 02/05/2014 ver 6.4 by Makoto. Help button added. Limitation and caution added.
% 06/28/2013 ver 6.3 by Makoto. Added fprintf('Saved %s/%s_backproj\n', pwd, dataName)
% 06/18/2013 ver 6.2 by Makoto. Clear STUDY.backproj timing optimized.
% 01/28/2013 ver 6.1 by Makoto. Combined conditions supported.
% 01/17/2013 ver 6.0 by Makoto. Bug fixed (thanks to Anthony Rissling).
% 01/04/2013 ver 5.1 by Makoto. Help added.
% 01/04/2013 ver 5.0 by Makoto. Handles multi-channel multi-cluser requests. Backprojected dataset can be saved.
% 10/26/2012 ver 4.0 by Makoto. Renewed for a plugin.
% 10/24/2012 ver 3.2 by Makoto. clear tmp actualindex added.
% 10/23/2012 ver 3.1 by Makoto. icList included in the final output. icList1 now refers to STUDY.design.
% 10/12/2012 ver 3.0 by Makoto. Multiple cluster backprojection supported
% 07/12/2012 ver 2.0 by Makoto. Serious bug fixed (channel number was used for IC indICes!) 
% 07/10/2012 ver 1.0 by Makoto. Test version created.

% Copyright (C) 2012 Makoto Miyakoshi JSPS/SCCN,INC,UCSD
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General PublIC LICense as published by
% the Free Software Foundation; either version 2 of the LICense, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General PublIC LICense for more details.
%
% You should have received a copy of the GNU General PublIC LICense
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1.07  USA

function [STUDY, backProjectedData, times, channel, cluster, uniqSubjGroupList, completeList] = std_backproj(STUDY, ALLEEG, EEG)

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% collect user input %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%
userInput = inputgui('title', 'std_ICbackprojection()', 'geom', ...
   {{2 7 [0 0] [1 1]}   {2 7 [1 0] [1 1]} ...
    {2 7 [0 1] [1 1]}   {2 7 [1 1] [1 1]} ...
    {2 7 [0 2] [1 1]}   {2 7 [1 2] [1 1]} ...
    {1 7 [0 3] [1 1]}  ...  
    {1 7 [0 4] [1 1]}  ...
    {1 7 [0 5] [1 1]}  ...
    {1 7 [0 6] [1 1]}  ... 
    {1 7 [0 7] [1 1]}  ... 
    {6 7 [0 8] [1 1]}},...
'uilist',...
   {{'style' 'text' 'string' 'Cluster number(s) to backproject. Ex. [1 2 7] or [1:3] '} {'style' 'edit' 'string' '[]'}  ...
    {'style' 'text' 'string' 'Channel(s) to be backprojected. Ex. Fz Cz Pz'}            {'style' 'edit' 'string' ''}  ...
    {'style' 'text' 'string' 'Else, just save IC-selected dataset(s)?'}                 {'style' 'checkbox' 'value' 0}...
    {'style' 'text' 'string' ''} ...
    {'style' 'text' 'string' '++++++++++++++  Within-subject conditions should be contained in one setfile. Session not supported yet.'} ...
    {'style' 'text' 'string' '+++ Limitations  +++  If check box on, IC-selected setfiles will be saved instead of ERP backprojection.'} ...
    {'style' 'text' 'string' '+++ & Cautions! +++  Backprojected data are located at STUDY.backProj.backProjectedData where {groups} and '} ...
    {'style' 'text' 'string' '++++++++++++++  [channel, data, subjects, conditions] that corresponds to STUDY.backProj.backProjSubjList'} ...
    {'style', 'pushbutton', 'string', 'Help', 'callback', 'pophelp(''std_backproj'');'}});

% if canceled, escape
if isempty(userInput)
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% remove existing STUDY.backProj %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(STUDY, 'backProj')
    STUDY = rmfield(STUDY, 'backProj');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% convert user input to usable variables %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clustNum     = eval(userInput{1,1});
backProjChan = strsplit(userInput{1,2}, ' ');
saveFlag     = userInput{1,3};

%%%%%%%%%%%%%%%%%%%%%
%%% create a list %%%
%%%%%%%%%%%%%%%%%%%%%
perGroupLength = zeros(1,length(STUDY.group));
for group = 1:length(STUDY.group)
    for clust = 1:length(clustNum)
        tmpLength      = length(STUDY.cluster(1,clustNum(clust)).setinds{1,group});
        perGroupLength(1,group) = perGroupLength(1,group) + tmpLength;
    end
end

for n = 1:length(STUDY.group)
    allSetIc{1,n} = zeros(perGroupLength(1,n), 2); 
end

for group = 1:length(STUDY.group)
    listCounter = 0;
    for clust = 1:length(clustNum)
        tmpSetInds = STUDY.cluster(1,clustNum(clust)).setinds{1,group};
        for m = 1:length(tmpSetInds)
            listCounter = listCounter+1;
            allSetIc{1,group}(listCounter,1) = STUDY.design(STUDY.currentdesign).cell(tmpSetInds(m)).dataset;            %#ok<*AGROW> % dataset number
            allSetIc{1,group}(listCounter,2) = STUDY.cluster(1,clustNum(clust)).allinds{1,group}(1,m); % IC number
        end
    end
end

% sort SetInds & create unique subject list
completeList = zeros(sum(perGroupLength),3);

for n = 1:length(STUDY.group)
    tmpSetInds    = allSetIc{1,n}(:,1);
    [~,sortIndex] = sort(tmpSetInds);
    allSetIc{1,n} = allSetIc{1,n}(sortIndex,:);
    
    if n == 1
        currentIndex = size(allSetIc{1,n},1);
        completeList(1:currentIndex,1) = allSetIc{1,n}(:,1);
        completeList(1:currentIndex,2) = allSetIc{1,n}(:,2);
        completeList(1:currentIndex,3) = repmat(n, [size(allSetIc{1,n},1) 1]);
        oldIndex = currentIndex;
    else
        currentIndex = size(allSetIc{1,n},1);
        completeList(oldIndex+1:oldIndex+currentIndex,1) = allSetIc{1,n}(:,1);
        completeList(oldIndex+1:oldIndex+currentIndex,2) = allSetIc{1,n}(:,2);
        completeList(oldIndex+1:oldIndex+currentIndex,3) = repmat(n, [size(allSetIc{1,n},1) 1]);
        oldIndex = oldIndex + currentIndex;
    end
end
[~,IA]  = unique(completeList(:,1));
uniqueSubj = completeList(IA,:);
uniqueSubj = uniqueSubj(:,[1 3]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% process each subject %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
waitBarHandle = waitbar(0,'Cluster ICs are now being backprojected...');
waitBarCount  = 0;

for group = 1:length(STUDY.group)
    perGroupIndex = find(uniqueSubj(:,2) == group);
    for n = 1:length(perGroupIndex)
        
        waitBarCount = waitBarCount+1;
        waitbar(waitBarCount/size(uniqueSubj,1), waitBarHandle)        
        
        tmpSetID = uniqueSubj(perGroupIndex(n));
        
        if n == 1
            [ALLEEG, EEG, CURRENTSET] = pop_newset(ALLEEG, EEG, 1, 'retrieve', tmpSetID, 'study', 1);
        else
            [ALLEEG, EEG, CURRENTSET] = pop_newset(ALLEEG, EEG, uniqueSubj(perGroupIndex(n-1)), 'retrieve', uniqueSubj(perGroupIndex(n)), 'study', 1);
        end
        
        fileName = ALLEEG(CURRENTSET).filename;
        dataName = fileName(1:end-4);
        backProj_subj{1,group}{n,1} = dataName;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% select listed ICs for this subject %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
        % find IC number to ramain
        tmpICindex = completeList(:,1) == CURRENTSET;
        tmpIC      = completeList(tmpICindex, 2);
        
        % reject non-listed ICs
        rejectIC = setdiff(1:size(EEG.icaweights,1), tmpIC);
        if ~isempty(rejectIC)
            if length(rejectIC) ~= size(EEG.icaweights,1);
                EEG = pop_subcomp(EEG, rejectIC, 0);
                skipSaveFlag = 0;
            else
                disp(['None of ICs from ' dataName ' is included in the selected clusters.'])
                skipSaveFlag = 1;
            end
        else
                skipSaveFlag = 0;
        end
        clear ind tmpIC rejectIC
        
%%%%%%%%%%%%%
%%% save? %%%
%%%%%%%%%%%%%
        if saveFlag == 1
            if skipSaveFlag ~= 1
                EEG = pop_saveset(EEG, 'filename', [dataName '_backproj'], 'filepath', pwd);
                fprintf('Saved %s/%s_backproj\n', pwd, dataName)
            end
        else

%%%%%%%%%%%%%%%%%%%%%%%%%
%%% finding a channel %%%
%%%%%%%%%%%%%%%%%%%%%%%%%
            tmpBackProjChan = zeros(length(backProjChan),1);
            for i = 1:length(backProjChan)
                tmpBackProjChan(i) = find(strcmp({EEG.chanlocs.labels}, backProjChan{i}));
            end            
            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Separate within-subject conditions %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            for m = 1:length(STUDY.design(STUDY.currentdesign).variable(1,1).value)
                tmpCondition = STUDY.design(STUDY.currentdesign).variable(1,1).value(1,m);
                if iscell(tmpCondition)
                    tmpEEG = pop_selectevent( EEG, 'type', tmpCondition{1,1}, 'deleteevents','off','deleteepochs','on','invertepochs','off');
                else
                    tmpEEG = pop_selectevent( EEG, 'type', tmpCondition,      'deleteevents','off','deleteepochs','on','invertepochs','off');
                end
                tmpEEG = squeeze(mean(tmpEEG.data, 3));
                tmpBackProj(:,:,n,m) = tmpEEG(tmpBackProjChan',:);
            end
            clear tmpCondition tmpEEG
        end
    end
    if saveFlag ~= 1
        backProj{1,group} = tmpBackProj;
        clear tmpBackProj
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% if saveFlag == 1, return %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if saveFlag == 1
    close(waitBarHandle)
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Store the results under STUDY.backProj %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for group = 1:length(backProj) % to read the number of groups
    for condition = 1:size(backProj{1,group}, 4)
        tmp = squeeze(backProj{1,group}(:,:,:,condition));
        erpData{condition, group} = tmp;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Store the results under STUDY.backProj %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% to store to STUDY
STUDY.backProj.backProjectedData = erpData;
STUDY.backProj.backProjSubjList  = backProj_subj;
STUDY.backProj.channel           = backProjChan;
STUDY.backProj.cluster           = clustNum;
STUDY.backProj.completeList      = completeList;
STUDY.backProj.uniqSubjGroupList = uniqueSubj;
STUDY.backProj.times             = EEG.times;



% to output - other than completeList
backProjectedData = erpData;
channel           = backProjChan;
cluster           = clustNum;
uniqSubjGroupList = uniqueSubj;
times             = EEG.times;

close(waitBarHandle)



function strings = strsplit(string, adjoiner) % Copyright 2008-2009 Levente Hunyadi

if nargin < 2
    adjoiner = sprintf('\n');
end

ix = strfind(string, adjoiner);
strings = cell(numel(ix)+1, 1);
ix = [0 ix numel(string)+1];  % place implICit adjoiners before and after string
for k = 2 : numel(ix)
    strings{k-1} = string(ix(k-1)+1:ix(k)-1);
end